Package: voi
Title: Expected Value of Information
Version: 1.0.3
Date: 2024-09-16
Authors@R: 
    c(person(given = "Christopher", family = "Jackson",role = c("aut", "cre"),email = "chris.jackson@mrc-bsu.cam.ac.uk"),
	   person(given = "Anna", family = "Heath", role = c("aut"), email = "anna.heath@sickkids.ca"),
	   person(given = "Gianluca", family = "Baio", role = c("ctb"), email = "g.baio@ucl.ac.uk", comment="Author of code taken from the BCEA package"),
	   person(given = "Mark", family = "Strong", role = c("ctb"), email = "mark.strong@sheffield.ac.uk", comment="Author of code taken from the SAVI package"),
	   person(given="Kofi Placid", family="Adragni", role="ctb", comment="Author of code taken from the ldr package"),
	   person(given="Andrew", family="Raim", role="ctb",comment="Author of code taken from the ldr package")
	   )
Description: Methods to calculate the expected value of information from a decision-analytic model.  This includes the expected value of perfect information (EVPI), partial perfect information (EVPPI) and sample information (EVSI), and the expected net benefit of sampling (ENBS).  A range of alternative computational methods are provided under the same user interface.  See Heath et al. (2024) <doi:10.1201/9781003156109>, Jackson et al. (2022) <doi:10.1146/annurev-statistics-040120-010730>.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0)
VignetteBuilder: knitr
Imports: mgcv, earth, mvtnorm, progress, dbarts, posterior, ggplot2,
        gridExtra, Matrix
Suggests: testthat, BCEA, INLA, splancs, sf, knitr, rmarkdown, rjags,
        truncnorm, scales, dplyr, heemod
RoxygenNote: 7.3.2
URL: https://chjackson.github.io/voi/
BugReports: https://github.com/chjackson/voi/issues
Additional_repositories: https://inla.r-inla-download.org/R/stable/
NeedsCompilation: no
Packaged: 2024-09-16 09:42:13 UTC; Chris J
Author: Christopher Jackson [aut, cre],
  Anna Heath [aut],
  Gianluca Baio [ctb] (Author of code taken from the BCEA package),
  Mark Strong [ctb] (Author of code taken from the SAVI package),
  Kofi Placid Adragni [ctb] (Author of code taken from the ldr package),
  Andrew Raim [ctb] (Author of code taken from the ldr package)
Maintainer: Christopher Jackson <chris.jackson@mrc-bsu.cam.ac.uk>
Repository: CRAN
Date/Publication: 2024-09-16 11:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 05:58:25 UTC; unix
