# Generated by roxygen2: do not edit by hand

S3method(plot,evppi)
export(all_interactions)
export(check_regression)
export(chemo_model_cea)
export(chemo_model_lor_cea)
export(chemo_model_lor_nb)
export(chemo_model_nb)
export(chemo_pars_fn)
export(enbs)
export(enbs_opt)
export(evpi)
export(evppi)
export(evppi_mc)
export(evppivar)
export(evsi)
export(evsivar)
export(import_heemod_inputs)
export(import_heemod_outputs)
export(pop_voi)
importFrom(Matrix,nearPD)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_linerange)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grDevices,dev.off)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(gridExtra,grid.arrange)
importFrom(progress,progress_bar)
importFrom(stats,AIC)
importFrom(stats,IQR)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cov)
importFrom(stats,dbinom)
importFrom(stats,dist)
importFrom(stats,dnorm)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rbinom)
importFrom(stats,reorder)
importFrom(stats,residuals)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(utils,combn)
importFrom(utils,select.list)
