# The Chemotherapy Model for "Value of Information for Health Economic Evalulations", Heath, Kunst and Jackson (2021).

This model is structured based on "A Need for Change! A Coding Framework for Improving Transparency in Decision Modeling" by Alarid-Escudero, F., Krijkamp, E., Pechlivanoglou, P., Jalal, H., Kao, S., Yang, A. and Enns, E.

This model has been adapted from a model presented in "Calculating the Expected Value of Sample Information Using Efficient Nested Monte Carlo: A Tutorial" by Heath, A. and Baio G. This will be used as the running example throughout the book.
